<?php
/**
 * Filtres SPIP fournis par le plugin
 *
 * @author		Jonathan OCHEJ
 * @license		MIT - https://github.com/GouvernementFR/dsfr/blob/main/LICENSE.md
 * 
 **/
if ( !defined('_ECRIRE_INC_VERSION') ) {
	return;
}

/**
 * Filtre `|dsfr_id`
 * 
 * Transforme un texte en `id` pouvant être utilisé
 * en toute sécurité pour l'attribut `id` d'une balise HTML
 *
 * @example
 *     `<balise[ id="(#TEXTE**|dsfr_id)"]></balise>`
 * 
 * @param string $texte
 *     Texte à transformer en `id`
 * 
 * @return string $id
 **/
function filtre_dsfr_id_dist($texte) {
	if ( empty($texte) ) {
		return '';
	}

	$separateur = '_';

	if ( !function_exists('translitteration') ) {
		include_spip('inc/charsets');
	}

	// pas de balise html
	if ( strpos($texte, '<') !== false ) {
		$texte = strip_tags($texte);
	}
	if ( strpos($texte, '&') !== false ) {
		$texte = unicode2charset($texte);
	}

	// supprime les espaces indésirables
	$texte = trim($texte);

	// enlève les accents et cie
	$texte = translitteration($texte);

	// remplace tout les caractères invalide par un séparateur
	$texte = preg_replace('/[^A-Za-z0-9\-]/', $separateur, $texte);

	// nettoyer les doubles occurences du separateur
	$texte = preg_replace('/'.$separateur.'+/', $separateur, $texte);

	// pas de separateur au debut ni a la fin
	$texte = trim($texte, $separateur);

	// est-ce que le premier caractère est numérique ?
	if ( !empty($texte) && preg_match('/[^a-z]/i', substr($texte, 0, 1)) ) {
		$texte = 'id_'.$texte; // un id doit toujours commencer par un caractère alphanumérique
	}

	return $texte;
}

/**
 * Filtre `|dsfr_class`
 * 
 * Transforme un tableau ou une chaine de texte en `nom_de_classe_css` pouvant être utilisé
 * en toute sécurité pour l'attribut `class` d'une balise HTML.
 *
 * @example
 *     `<balise[ class="(#TEXTE|dsfr_class)"]></balise>`
 *     `<balise[ class="(#LISTE{nom_de_classe_css,autre_nom_de_classe_css}|dsfr_class)"]></balise>`
 *     `<balise[ class="(#VAL{nom_de_classe_css autre_nom_de_classe_css}|dsfr_class)"]></balise>`
 * 
 * @param string|array $classes_css
 * 
 * @return string $classes_css
 **/
function filtre_dsfr_class_dist($classes_css) {

	if ( is_string($classes_css) ) {
		$classes_css = explode(' ',trim($classes_css));
	}

	if ( is_array($classes_css) ) {
		$classes_css = array_map('trim', $classes_css);
		$classes_css = array_filter($classes_css);
		$classes_css = array_unique($classes_css);
	}

	if ( empty($classes_css) ) {
		return '';
	}

	return is_array($classes_css) ? implode(' ', array_map('attribut_html', $classes_css)) : attribut_html($classes_css);
}

/**
 * Filtre `|dsfr_class_ajouter{nom_de_classe_css}`
 * 
 * Ajoute un ou plusieurs `nom_de_classe_css` dans un tableau.
 * Si `nom_de_classe_css` existe déjà dans le tableau, le tableau n'est pas modifié.
 *
 * @example
 *     `#ARRAY|dsfr_class_ajouter{nom_de_classe_css}`
 * 
 *     `#SET{class,#ARRAY}
 *      #SET{class,#GET{class}|dsfr_class_ajouter{nom_de_classe_css}}`
 * 
 *     `#ARRAY|dsfr_class_ajouter{nom_de_classe_css autre_nom_de_classe_css}`
 *     `#ARRAY|dsfr_class_ajouter{#LISTE{nom_de_classe_css,autre_nom_de_classe_css}}`
 * 
 * @param array $tableau_de_classes_css
 * @param string|array $classes_css
 * 
 * @return array $tableau_de_classes_css
 **/
function filtre_dsfr_class_ajouter_dist($tableau_de_classes_css, $classes_css) {
	if ( !is_array($tableau_de_classes_css) ) {
		return $tableau_de_classes_css;
	}

	if ( is_string($classes_css) ) {
		$classes_css = explode(' ',trim($classes_css));
	}

	if ( is_array($classes_css) ) {
		$classes_css = array_map('trim', $classes_css);
		$classes_css = array_filter($classes_css);
		$classes_css = array_unique($classes_css);
	}

	if ( !is_array($classes_css) || empty($classes_css) ) {
		return $tableau_de_classes_css;
	}

	$tableau_de_classes_css = array_merge($tableau_de_classes_css, $classes_css);

	return array_values(array_unique($tableau_de_classes_css));
}

/**
 * Filtre `|dsfr_class_supprimer{nom_de_classe_css}`
 * 
 * Supprime un ou plusieurs `nom_de_classe_css` dans un tableau.
 * Si `nom_de_classe_css` n'existe pas dans le tableau, le tableau n'est pas modifié.
 *
 * @example
 *     `#ARRAY|dsfr_class_supprimer{nom_de_classe_css}`
 * 
 *     `#SET{class,#ARRAY}
 *      #SET{class,#GET{class}|dsfr_class_ajouter{nom_de_classe_css}}
 *      #SET{class,#GET{class}|dsfr_class_supprimer{nom_de_classe_css}}`
 * 
 *     `#ARRAY|dsfr_class_supprimer{nom_de_classe_css autre_nom_de_classe_css}`
 *     `#ARRAY|dsfr_class_supprimer{#LISTE{nom_de_classe_css,autre_nom_de_classe_css}}`
 * 
 * @param array $tableau_de_classes_css
 * @param string|array $classes_css
 * 
 * @return array $tableau_de_classes_css
 **/
function filtre_dsfr_class_supprimer_dist($tableau_de_classes_css, $classes_css) {
	if ( !is_array($tableau_de_classes_css) ) {
		return $tableau_de_classes_css;
	}

	if ( is_string($classes_css) ) {
		$classes_css = explode(' ',trim($classes_css));
	}

	if ( is_array($classes_css) ) {
		$classes_css = array_map('trim', $classes_css);
		$classes_css = array_filter($classes_css);
		$classes_css = array_unique($classes_css);
	}

	if ( !is_array($classes_css) || empty($classes_css) ) {
		return $tableau_de_classes_css;
	}

	return array_values(array_diff($tableau_de_classes_css, $classes_css));
}

/**
 * Filtre `|dsfr_attributs`
 * 
 * Transforme un tableau en attributs HTML pouvant être utilisés
 * en toute sécurité avec une balise HTML.
 *
 * @example
 *     `<balise[ (#ARRAY|dsfr_attributs)]></balise>`
 * 
 * @param array $tableau_attributs_html
 * 
 * @return string $attributs_html
 **/
function filtre_dsfr_attributs_dist($tableau_attributs_html) {
	if ( !is_array($tableau_attributs_html) || empty($tableau_attributs_html) ) {
		return '';
	}

	$attributs_html = [];
	foreach ( $tableau_attributs_html as $cle => $valeur ) {
		$valeur = trim($valeur);
		if ( !empty($valeur) ) {
			if ( $valeur == 'booleen' )	{
				$attributs_html[] = $cle;
			}
			// ne pas enlever les balises html pour l'attribut `placeholder`
			else if ( $cle == 'placeholder' ) {
				$attributs_html[] = $cle.'="'.attribut_html($valeur, false).'"';
			}
			else {
				$attributs_html[] = $cle.'="'.attribut_html($valeur).'"';
			}
		}
	}

	return !empty($attributs_html) ? implode(' ', $attributs_html) : '';
}

/**
 * Filtre `|dsfr_attribut_definir{nom_attribut_html,valeur}`
 * 
 * Définit la `valeur` de `nom_attribut_html` dans un tableau.
 * Si `nom_attribut_html` existe déjà dans le tableau, sa `valeur` est modifiée par la nouvelle.
 * 
 * Si `nom_attribut_html` ou `valeur` est vide, le tableau n'est pas modifié.
 * 
 * Si `valeur=booleen` alors `nom_attribut_html` sera considéré comme un booléen lors de l'appel du filtre. `|dsfr_attributs`
 *
 * @example
 *     `#ARRAY|dsfr_attribut_definir{nom_attribut_html,valeur}`
 *     `#ARRAY|dsfr_attribut_definir{nom_attribut_html,booleen}`
 * 
 *     `#SET{attributs,#ARRAY}
 *      #SET{attributs,#GET{attributs}|dsfr_attribut_definir{nom_attribut_html,valeur}}`
 * 
 * @param array $tableau_attributs_html
 * @param string $attribut
 * @param string $valeur
 * 
 * @return array $tableau_attributs_html
 **/
function filtre_dsfr_attribut_definir_dist($tableau_attributs_html, $attribut, $valeur) {
	$attribut = trim($attribut);
	$valeur = trim($valeur);

	if ( !is_array($tableau_attributs_html) || empty($attribut) || empty($valeur) ) {
		return $tableau_attributs_html;
	}

	$tableau_attributs_html[$attribut] = $valeur;

	return $tableau_attributs_html;
}

/**
 * Filtre `|dsfr_attribut_supprimer{nom_attribut_html}`
 * 
 * Supprime `nom_attribut_html` du tableau.
 *
 * @example
 *     `#ARRAY|dsfr_attribut_supprimer{nom_attribut_html}`
 * 
 *     `#SET{attributs,#ARRAY}
 *      #SET{attributs,#GET{attributs}|dsfr_attribut_definir{nom_attribut_html,valeur}}
 *      #SET{attributs,#GET{attributs}|dsfr_attribut_supprimer{nom_attribut_html}}`
 * 
 * @param array $tableau_attributs_html
 * @param string $attribut
 * 
 * @return array $tableau_attributs_html
 **/
function filtre_dsfr_attribut_supprimer_dist($tableau_attributs_html, $attribut) {
	$attribut = trim($attribut);

	if ( !is_array($tableau_attributs_html) || empty($attribut) ) {
		return $tableau_attributs_html;
	}

	if ( array_key_exists($attribut,$tableau_attributs_html) ) {
		unset($tableau_attributs_html[$attribut]);
	}

	return $tableau_attributs_html;
}

/**
 * Filtre `|dsfr_texte_paragraphe{balises_a_conserver}`
 * 
 * Supprime toutes les balises html d'un texte sauf '<a><span><strong><i><sup><sub>' et
 * transforme les `<p>` en saut de ligne `<br>`.
 * 
 * Utile pour facilement convertir un contenu qui doit être inséré dans un paragraphe `<p>` 
 *
 * @example
 *     `#VAL{<h1>Titre</h1><div>texte</div>}|dsfr_texte_paragraphe` => `Titre texte`
 *     `#VAL{<h1>Titre</h1><p>texte <img src="..."></p>}|dsfr_texte_paragraphe{<img>}` => `Titre <br> texte <img src="...">`
 * 
 * @param string $texte
 * @param string $balises_a_conserver
 * 
 * @return string $texte
 **/
function filtre_dsfr_texte_paragraphe($texte, $balises_a_conserver = '<a><span><strong><i><sup><sub>') {
	if ( $texte === null || $texte === '' ) {
		return '';
	}

	// ajoute un espace pour éviter de transformer `<h1>Titre</h1><p>texte</p>` en `Titretexte`
	$texte = str_replace('<', ' <', $texte);

	// supprime les balises en conservant aussi les `<p>` et `<br>`
	$texte = strip_tags($texte, $balises_a_conserver.'<p><br>');

	// conversion des `<p>` en double saut de ligne `<br>` (simule visuellement un paragraphe)
	// l'utilisation de `PtoBR($texte)` ne convenait pas car elle ne donne pas le même effet en remplaçant les paragraphes par 1 seul saut de ligne
	$texte = preg_replace('/<p[^>]*?>/', '', $texte);
	$texte = str_replace('</p>', '<br><br>', $texte);  // double saut de ligne `<br>`
	$texte = preg_replace('/(<br>)+$/', '', $texte); // supprime les `<br>` à la fin

	// supprime les espaces en doubles
	$texte = preg_replace('/\s+/', ' ', $texte);

	return trim($texte);
}